#include <qmessagebox.h>
#include <qimage.h>
#include <qfiledialog.h>
#include <list>
#include <iostream> 
#include <fstream>

#include "viewer.h"

using namespace qglviewer;
using namespace std;

Viewer::Viewer(QWidget *parent, const char *name) : QGLViewer(parent,name),m(),drawMesh_(false)
{
};

void Viewer::draw()
{
  
  // Save the current model view matrix (not needed here in fact)
  glPushMatrix();
  // Multiply matrix to get in the frame coordinate system.
  glMultMatrixd(manipulatedFrame()->matrix());
  setHandlerKeyboardModifiers(QGLViewer::FRAME, Qt::AltButton);
  setHandlerKeyboardModifiers(QGLViewer::CAMERA, Qt::NoButton);
  setHandlerKeyboardModifiers(QGLViewer::FRAME, Qt::ControlButton);
  
  if (drawMesh_) {
  	// draw polygons
	glColor3f(0.0,0.5,0.8); // bleu turquoise
    	
	// let the mesh update the normals
    	m.update_normals();
		
	for (MyMesh::ConstFaceIter f_it = m.faces_begin(); f_it != m.faces_end(); ++f_it) {
		// begin polygon assembly
		glBegin(GL_POLYGON);
			
		// one normal per face
		const MyMesh::Normal& normal = m.normal( f_it );
		if (invertNormals_) glNormal3f(-normal[0],-normal[1],-normal[2]);
		else glNormal3f(normal[0],normal[1],normal[2]);
	
		// revolve around current face to get vertices
		for (MyMesh::ConstFaceVertexIter fv_it = m.fv_iter(f_it.handle()); fv_it; ++fv_it) {
			// one normal per vertex
 			if(smooth_) {
				const MyMesh::Normal& normal = m.normal( fv_it );
 				if (invertNormals_) glNormal3f(-normal[0],-normal[1],-normal[2]);
				else glNormal3f(normal[0],normal[1],normal[2]);
			}

			// polygon assembly is performed per vertex
 			const MyMesh::Point& point  = m.point(fv_it);
 			glVertex3d(point[0],point[1],point[2]);
		}
		
		// end polygon assembly	
		glEnd();
	}
	glFlush(); 
	
	if (drawEdges_) {
		// Surimpose edges
		glColor3f(0.0,0.0,0.0); // noir
		glBegin(GL_LINES);
		for (MyMesh::ConstEdgeIter e_it = m.edges_begin(); e_it != m.edges_end(); ++e_it) {
 			// assembly and draw line segment
			const MyMesh::HalfedgeHandle& he=m.halfedge_handle(e_it.handle(),0);
	               	const MyMesh::Point& vha=m.point(m.from_vertex_handle(he));
	                const MyMesh::Point& vhb=m.point(m.to_vertex_handle(he));
			glVertex3f(vha[0],vha[1],vha[2]);
			glVertex3f(vhb[0],vhb[1],vhb[2]);
   		 }
		glEnd();
	}
  }
 
  // Restore the original (world) coordinate system
  glPopMatrix();
 
};


//   D i s p l a y   e v e n t s

Viewer::DisplayMode Viewer::dispMode() const
{
	return displayMode_;
};


void Viewer::dispMode(DisplayMode displaymode)
{	
	displayMode_ = displaymode;
	glFlush();
};


void Viewer::toggleDrawEdges()
{
	drawEdges_ = !drawEdges_;
	updateGL();
}

void Viewer::toggleInvertNormals()
{
	invertNormals_ = !invertNormals_;
	updateGL();
}

void Viewer::toggleDisplayRep()
{
	if (displayMode_ == SURF) {
		dispMode(WIRE);
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	}
  	else {
		if (displayMode_ == WIRE) {
			dispMode(PTS);
			glPolygonMode(GL_FRONT_AND_BACK, GL_POINT);
		}
  		else {
			dispMode(SURF);
			glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
		}
  	}
  	updateGL();
}

void Viewer::toggleSmoothShading()
{
	smooth_ = !smooth_;
	updateGL();
}

void Viewer::init()
{
  glClearColor(1.0,1.0,1.0,1.0);
  glDisable(GL_CULL_FACE);
  
  #ifdef GL_RESCALE_NORMAL  // OpenGL 1.2 Only...
  glEnable(GL_RESCALE_NORMAL);
  #endif
  // Add a manipulated frame to the viewer.
  setManipulatedFrame(new ManipulatedFrame());
  	
  // Enable GL textures
  glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
  glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
  // Nice texture coordinate interpolation
  glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );
  
  drawEdges_ = false;
  invertNormals_ = false;
  smooth_ = true;
  dispMode(SURF);

  // Add custom key description (see keyPressEvent)
  setKeyDescription(Key_E, "Toggles the display of edges.");
  setKeyDescription(Key_N, "Inverts the face normals.");
  setKeyDescription(Key_R, "Sets surface, wireframe or points representation.");
  setKeyDescription(Key_X, "Toggles smooth shading.");
  setKeyDescription(Key_Return, "Saves a screenshot.");
  setKeyDescription(CTRL+Key_O, "Opens a file.");
  setKeyDescription(CTRL+Key_S, "Saves scene in a file.");
 
};

void Viewer::changeMesh(const char* filename)
{
	drawMesh_ = false;
	
	// request vertex normals, so the mesh reader can use normal information
  	// if available
  	m.request_vertex_normals();
	
	// assure we have vertex normals
  	if (!m.has_vertex_normals()) {
    		cerr << "ERROR: Standard vertex property 'Normals' not available!\n";
    		return;
 	}
	
	OpenMesh::IO::Options opt;
  	if ( ! OpenMesh::IO::read_mesh(m, filename, opt)) {
    		cerr << "Error loading mesh from file " << filename << endl;
    		return;
  	}

  	// If the file did not provide vertex normals, then calculate them
  	if ( !opt.check( OpenMesh::IO::Options::VertexNormal ) ) {
    		// we need face normals to update the vertex normals
    		m.request_face_normals();

    		// let the mesh update the normals
    		m.update_normals();
		
  	}
	OpenMesh::IO::read_mesh(m, filename);
	
	drawMesh_ = true;
	
	// Calcul des coordonnes du centre et rayon de la bote englobante
	double center[3], diameter, w, h, k;
	
  	MyMesh::ConstVertexIter v_it = m.vertices_begin();
  	double min[3] = { (m.point(v_it))[0],(m.point(v_it))[1],(m.point(v_it))[2] };
  	double max[3] = { (m.point(v_it))[0],(m.point(v_it))[1],(m.point(v_it))[2] };
	
  	for (MyMesh::ConstVertexIter v_it = m.vertices_begin(); v_it != m.vertices_end(); ++v_it) {
		if ((m.point(v_it))[0] > max[0]) max[0] = (m.point(v_it))[0];
      		if ((m.point(v_it))[0] < min[0]) min[0] = (m.point(v_it))[0];
		if ((m.point(v_it))[1] > max[1]) max[1] = (m.point(v_it))[1];
      		if ((m.point(v_it))[1] < min[1]) min[1] = (m.point(v_it))[1];
		if ((m.point(v_it))[2] > max[2]) max[2] = (m.point(v_it))[2];
      		if ((m.point(v_it))[2] < min[2]) min[2] = (m.point(v_it))[2];
  	}
	
  	center[0] = (max[0]+min[0])/2.0; w = (max[0]-min[0])/2.0;
  	center[1] = (max[1]+min[1])/2.0; h = (max[1]-min[1])/2.0;
  	center[2] = (max[2]+min[2])/2.0; k = (max[2]-min[2])/2.0;
  	diameter = (w > h) ? w : h;
	if (k > diameter) diameter = k;
	
  	if (diameter < 0.1) diameter = 0.1;
	
	// QGLViewer construit la scne  partir de la bote englobante
	camera()->setSceneCenter(Vec(center[0],center[1],center[2]));
  	camera()->setSceneRadius(diameter*2.0);
  	showEntireScene();
	
	glFlush();
};

void Viewer::saveMesh(const char* filename)
{
	try {
    		if ( !OpenMesh::IO::write_mesh(m, filename) ) {
			cerr << "Cannot write mesh to file " << filename << endl;
			return;
		}
    	}	
        catch( exception& x ) {
    		cerr << x.what() << endl;
	}
};

QString Viewer::helpString() const
{
  QString text("<h2>Interface projets image</h2>");
  return text;
};
